package com.muyu.marketing.team.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.utils.bean.BeanUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.marketing.domain.ActivityTeamInfo;
import com.muyu.marketing.domain.ActivityTeamProductSkuInfo;
import com.muyu.marketing.domain.model.*;
import com.muyu.marketing.domain.resp.ActivityTeamInfoDetailResp;
import com.muyu.marketing.domain.resp.ActivityTeamProductSkuResp;
import com.muyu.marketing.team.mapper.ActivityTeamInfoMapper;
import com.muyu.marketing.team.service.ActivityTeamInfoService;
import com.muyu.marketing.team.service.ActivityTeamOpenInfoService;
import com.muyu.marketing.team.service.ActivityTeamProductSkuInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;


@Service
public class ActivityTeamInfoServiceImpl extends ServiceImpl<ActivityTeamInfoMapper, ActivityTeamInfo>
        implements ActivityTeamInfoService {

    @Autowired
    private ActivityTeamOpenInfoService activityTeamOpenInfoService;

    @Autowired
    private ActivityTeamProductSkuInfoService activityTeamProductSkuInfoService;
//    @Autowired
//    private ActivityTeamInfoMapper activityTeamInfoMapper;


    @Override
    public TableDataInfo<ActivityTeamInfoListModel> query(ActivityTeamInfoListQueryModel activityTeamInfoListQueryModel) {

        LambdaQueryWrapper<ActivityTeamInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getKeyWord()), ActivityTeamInfo::getName, activityTeamInfoListQueryModel.getKeyWord());
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getStatus()), ActivityTeamInfo::getStatus, activityTeamInfoListQueryModel.getStatus());

        /**
         * Object<T> -> 创建对象的时候进行的占用
         * <T> Result<T> 以方法返回值为占用
         */
        Page<ActivityTeamInfo> activityTeamInfoPage = this.page(activityTeamInfoListQueryModel.buildPage(), queryWrapper);
        List<ActivityTeamInfo> activityTeamInfoList = activityTeamInfoPage.getRecords();
        List<ActivityTeamInfoListModel> activityTeamInfoListModels = activityTeamInfoList.stream()
                .map(activityTeamInfo -> ActivityTeamInfoListModel.infoBuild(activityTeamInfo,
                        (activityTeamInfoListModelBuilder) -> {
                            TeamProductDiscountPriceModel discountPrice = activityTeamProductSkuInfoService.getDiscountPrice(activityTeamInfo.getId());
                            TeamProductStockModel teamProductStockModel = activityTeamProductSkuInfoService.getStock(activityTeamInfo.getId());
                            Long teamOpenTypeNumber = activityTeamOpenInfoService.getTeamOpenTypeNumberByTeamId(activityTeamInfo.getId());
                            Long teamInTypeNumber = activityTeamOpenInfoService.getTeamInTypeNumberByTeamId(activityTeamInfo.getId());

                            return activityTeamInfoListModelBuilder
                                    .openTeamNumber(teamOpenTypeNumber)
                                    .addTeamNumber(teamInTypeNumber)
                                    .attendNumber(teamOpenTypeNumber + teamInTypeNumber)
                                    .teamPrice(discountPrice.getTeamPrice())
                                    .productPrice(discountPrice.getProductPrice())
                                    .teamStock(teamProductStockModel.getTeamStock())
                                    .remainStock(teamProductStockModel.getRemainStock())
                                    .build();
                        })).toList();
        TableDataInfo<ActivityTeamInfoListModel> tableDataInfo = new TableDataInfo<>();
        tableDataInfo.setTotal(activityTeamInfoPage.getTotal());
        tableDataInfo.setRows(activityTeamInfoListModels);
        return tableDataInfo;
    }

    /**
     * 通过添加模型，去进行商品拼团活动的操作
     *
     * @param activityTeamInfoAddModel 团购添加模型
     */
    @Override
    public void save(ActivityTeamInfoAddModel activityTeamInfoAddModel) {
        boolean save = this.save(ActivityTeamInfo.addModelBuild(activityTeamInfoAddModel));
        if (!save) {
            throw new ServiceException(
                    StringUtils.format("团购活动添加失败：[{}]", activityTeamInfoAddModel)
            );
        }
        this.activityTeamProductSkuInfoService.batchSave(
                activityTeamInfoAddModel.getActivityTeamProductSkuAddModelList()
        );
    }

    /**
     * 根据id查询活动详情
     *
     * @param teamId
     * @return
     */
    @Override
    public ActivityTeamInfoDetailResp getActivityByTeamId(Long teamId) {
        //根据id查询活动详情
        ActivityTeamInfo activityTeamInfo = getById(teamId);
        //封装活动详情
        ActivityTeamInfoDetailResp activityTeamInfoDetailResp = new ActivityTeamInfoDetailResp();
        //根据活动id查询活动商品SKU集合
        List<ActivityProductSkuListModel> teamProductSkuInfoList = activityTeamProductSkuInfoService.getTeamProductSkuInfoListByTeamId(teamId);
        if (teamProductSkuInfoList != null) {
            //定义一个空集合
            ArrayList<ActivityTeamProductSkuResp> activityProductSkuList = new ArrayList<>();
            for (ActivityProductSkuListModel activityProductSkuListModel : teamProductSkuInfoList) {
                Long id = activityProductSkuListModel.getId();
                //根据id查询
                ActivityTeamProductSkuInfo activityTeamProductSkuInfo = activityTeamProductSkuInfoService.getById(id);
                //添加到集合中
                activityProductSkuList.add(
                        ActivityTeamProductSkuResp.builder()
                                .ProductId(activityTeamProductSkuInfo.getProductId())
                                .sku(activityTeamProductSkuInfo.getProductSku())
                                .teamPrice(activityTeamProductSkuInfo.getTeamPrice())
                                .teamStock(activityTeamProductSkuInfo.getTeamStock())
                                .build()
                );
            }

            activityTeamInfoDetailResp.setActivityTeamProductSkuList(activityProductSkuList);
            BeanUtils.copyBeanProp(activityTeamInfoDetailResp, activityTeamInfo);
        }
        return activityTeamInfoDetailResp;
    }

}
